# -*- coding: utf-8 -*-

from zope.component import adapts
from zope.interface import implements

from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField

from Products.Archetypes.Field import LinesField
from Products.Archetypes import atapi

#from Products.ATContentTypes.interface import IATFolder
from cciaa.modulistica.interfaces import CCIAAModAbleContent
from cciaa.modulistica import modulisticaMessageFactory as _

class ExtensionColumnsField(ExtensionField, LinesField):
    """ derivative of LinesField for extending schemas """


class FolderWithColumnsExtender(object):
    adapts(CCIAAModAbleContent)
    implements(ISchemaExtender)

    fields = [
        ExtensionColumnsField('columns',
            widget=atapi.LinesWidget(
                label= _(u'label_cciaafolder_columnstitles', default=u'Columns titles'),
                description = _(u'help_cciaafolder_columnstitles',
                                default=u"Put there, one per line (up to 4), columns titles that must be put on the top of the "
                                        u"created tables when using the files views. \n"
                                        u"Not providing those values will not display any table headers.",
                ),
                condition="python:here.getLayout()=='cciaa_modulistica_view'",
            ),
            required=False,
        ),

    ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields



# -*- coding: utf-8 -*-

import os
from setuptools import setup, find_packages

version = '2.1.1'

tests_require=['zope.testing', 'Products.PloneTestCase']

setup(name='cciaa.modulistica',
      version=version,
      description="An additional view for Plone folders, for better manage repository of files",
      long_description=open("README.rst").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.0',
        'Framework :: Plone :: 4.1',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        ],
      keywords='plone plonegov form file',
      author='RedTurtle Technology',
      author_email='sviluppoplone@redturtle.it',
      url='http://plone.org/products/cciaa.modulistica',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['cciaa', ],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'Products.CMFPlone',
                        'archetypes.schemaextender',
                        ],
      tests_require=tests_require,
      extras_require=dict(test=tests_require),
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-

from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter

from cciaa.modulistica import modulisticaMessageFactory as _

class ModulisticaView(BrowserView):
    """View for seeing the files in the folder"""

    IMG_TAG = """<img src="%s" alt="%s" title="%s" />"""

    def getDocumentContentOf(self, doc_id):
        """given an id, load from this folder a page"""
        context = self.context
        page = getattr(context, doc_id)
        body = '<a name="section-'+ doc_id +'"></a>' + page.getText()
        return body

    def generateImgTag(self, icon, alt="", title=""):
        """Given an icon, generate one from (X)HTML to be used in the view"""
        src = icon
        if not src:
            # Can happens for some values of icon_visibility in site_properties
            return alt
        return self.IMG_TAG % (src, alt, title)

    def safe_unicode(self, value):
        if isinstance(value, unicode):
            return value
        elif isinstance(value, str):
            try:
                return unicode(value, 'utf-8')
            except UnicodeDecodeError:
                return unicode(value, 'utf-8', 'ignore')
        return str(value)

    def getDownloadMessage(self, title, type):
        """'download' or 'goto'"""
        if type=='download':
            return _('download_message',
                     default=u'Download the file ${title}',
                     mapping={'title': self.safe_unicode(title)})
        elif type=='goto':
            return _('goto_message',
                     default=u'Go to link ${title}',
                     mapping={'title': self.safe_unicode(title)})
        return None

    def generateColumns(self,related_items,num_rel):
        """Generate table columns"""
        var = related_items - num_rel
        stringa = ""
        for i in range(0, var):
            stringa += "<td class=\"col3\"/>"
        return stringa

    def count_related_items(self, items):
        """ return how many related items a file has, limited to 2"""
        max = 0
        for item in items:
            if item.portal_type == 'File':
                try:
                    rel_items = item.getObject().getRawRelatedItems()
                except AttributeError:
                    # the item doesn't have related items behavior
                    rel_items = 0
                if len(rel_items) > max:
                    max = len(rel_items)
            if max > 2:
                return 2
        return max

    def getColumns(self):
        return self.context.getField('columns').get(self.context)

    def getTitles(self, items):
        """
        Generate the table header with titles given in the folder new field
        """
        stringa = ""
        num_related = self.count_related_items(items) + 2
        columns = self.getColumns()

        if columns:
            if len(columns) > num_related:
                titles = columns[:num_related]
            else:
                titles = columns

            for title in titles:
                stringa += '<th class="nosort" scope="col">' + title + "</th>"
            if len(titles) < num_related:
                for i in range(num_related-len(titles)):
                    stringa += "<th>" "</th>"
        return """<tr class="headingModuli">%s</tr>""" % stringa

    def isRemote(self, url):
        """Check if the given URL is remote or not
        only HTTP or HTTPS can be remote URLs
        """
        if not url.lower().startswith("http"):
            return False
        portal_url = getToolByName(self.context, 'portal_url')()
        return not url.startswith(portal_url)

    def getExternalLinkMessage(self):
        return _(u'External link')

    def isAnon(self):
        return getToolByName(self.context, 'portal_membership').isAnonymousUser()

    def member(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name=u'plone_portal_state')
        return portal_state.member()



# -*- coding: utf-8 -*-

from cciaa.modulistica.tests.base import TestCase
import re

class TestView(TestCase):

    def afterSetUp(self):
        self.setRoles(('Manager', ))
        portal = self.portal
        portal.invokeFactory(type_name="Folder", id="modulistica")
        m = getattr(portal, 'modulistica')
        m.edit(title="Modulistica")
        m.manage_addProperty('layout', 'cciaa_modulistica_view', 'string')
        
    def getModulistica(self):
        return self.portal.modulistica

    def test_addFile(self):
        m = self.getModulistica()
        m.invokeFactory(type_name='File', id='f1')
        f1 = getattr(m, 'f1')
        f1.edit(title='File 1', file='Lorem ipsum')
        self.assertTrue('Download the file File 1' in m())

    def test_addFileWithUnicodeName(self):
        m = self.getModulistica()
        m.invokeFactory(type_name='File', id='f1')
        f1 = getattr(m, 'f1')
        f1.edit(title='File ùno', file='Lorem ipsum')
        self.assertTrue(u'Download the file File ùno' in m())

    def test_addInternalLink(self):
        m = self.getModulistica()
        m.invokeFactory(type_name='Link', id='l1')
        l1 = getattr(m, 'l1')
        l1.edit(title='Link 1', remoteUrl=self.portal['news'].absolute_url())
        result = m()
        self.assertTrue('Visita Link 1' in result)
        self.assertFalse('ollegamento esterno' in result)
        
    def test_addExternalLinkNoDescr(self):
        """When external link has no descr, show only the tip text"""
        m = self.getModulistica()
        m.invokeFactory(type_name='Link', id='l1')
        l1 = getattr(m, 'l1')
        l1.edit(title='Link 1', remoteUrl='http://www.plone.org/')
        result = m()
        self.assertTrue('Visita Link 1' in result)
        self.assertEquals(len(re.findall('xternal link',result)), 1)

    def test_addExternalLinkWithDescr(self):
        """When external link has also descr, show tip text and description with additional text"""
        m = self.getModulistica()
        m.invokeFactory(type_name='Link', id='l1')
        l1 = getattr(m, 'l1')
        l1.edit(title='Link 1', remoteUrl='http://www.plone.org/', description="Go to plone.org")
        result = m()
        self.assertTrue('Visita Link 1' in result)
        self.assertEquals(len(re.findall('xternal link',result)), 2)

    def test_noHTTPExternalLink(self):
        """All links that don't starts with HTTP(s) are not external"""
        m = self.getModulistica()
        m.invokeFactory(type_name='Link', id='l1')
        l1 = getattr(m, 'l1')
        l1.edit(title='Link 1', remoteUrl='mailto:plone-devel@plone.org', description="Write to plone")
        result = m()
        self.assertTrue('Visita Link 1' in result)
        self.assertFalse('external link' in result)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestView))
    return suite
